from django.apps import apps
from django.contrib.auth import get_user_model
from django.urls import reverse_lazy

from .settings import acc_settings

User = get_user_model()


class DefaultUserAdapter:
    name = "accounts"

    def get_app_list(self, user):
        app_list = []
        for app_config in apps.get_app_configs():
            if user.has_module_perms(app_config.label):
                app_dict = {
                    "name": app_config.verbose_name,
                    "app_label": app_config.label,
                    "app_url": reverse_lazy(f"{self.name}:home"),
                    "has_module_perms": user.has_module_perms(app_config.label),
                    'models': self.get_models_for_app(app_config, user),
                }
                app_list.append(app_dict)
        return app_list

    def get_models_for_app(self, app_config, user):
        models = []
        for model in app_config.get_models():
            if model == User:
                models.append(self.get_user_model(user))
        return models

    def get_user_model(self, user):
        return {
            "model": User,
            "name": User._meta.verbose_name.capitalize(),
            "admin_url": reverse_lazy(
                f"{self.name}:profile", kwargs={"pk": user.pk.__str__()}
            ),
            "view_only": False,
        }

    def get_client_ip(self, request):
        x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
        if x_forwarded_for:
            ip = x_forwarded_for.split(",")[0]
        else:
            ip = request.META.get("REMOTE_ADDR")
        return ip


def get_adapter():
    return acc_settings.ADAPTER()



from urllib.parse import quote as urlquote

from django.contrib.auth import get_user_model
from django.contrib.auth.decorators import login_required
from django.contrib.auth.views import LoginView
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.shortcuts import resolve_url, redirect
from django.urls import reverse, reverse_lazy
from django.utils.decorators import method_decorator
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from django.views.decorators.csrf import csrf_protect
from django.views.generic import UpdateView
from django.views.generic.base import TemplateView

from .adapter import get_adapter
from .forms import ProfileForm
from .settings import acc_settings

User = get_user_model()
adapter = get_adapter()


class BaseView:
    site_title = acc_settings.SITE_TITLE
    site_header = acc_settings.SITE_HEADER
    errors = None

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(
            title=self.title,
            site_title=self.site_title,
            site_header=self.site_header,
            errors=self.errors,
        )
        return context


class MyLoginView(BaseView, LoginView):
    template_name = "accounts/login_form.html"
    redirect_authenticated_user = True
    title = _("Login")

    def get_default_redirect_url(self):
        """Return the default redirect URL."""
        if self.next_page:
            return resolve_url(self.next_page)
        else:
            return resolve_url(acc_settings.LOGIN_REDIRECT_URL)


class HomeView(BaseView, TemplateView):
    name = "accounts"
    template_name = "accounts/home.html"
    title = _("Home")

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        data = {
            "subtitle": None,
            "messages": messages.get_messages(self.request),
            "app_list": adapter.get_app_list(self.request.user),
            "client_ip": adapter.get_client_ip(self.request),
            "ua_string": self.request.META["HTTP_USER_AGENT"],
            **(self.extra_context or {})
        }
        context.update(data)
        return context

    @method_decorator(csrf_protect)
    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super().dispatch(request, *args, **kwargs)


class ProfileView(BaseView, UpdateView):
    template_name = "accounts/profile_form.html"
    title = _("Profile")
    model = User
    form_class = ProfileForm
    app_label = User._meta.app_label

    fieldsets = (
        (_('Identity'), {'fields': ('username', 'password')}),
        (_('Personal infomation'), {
            'classes': ['collapse'],
            'fields': ('first_name', 'last_name', 'email'),
        }),
        (_('Important dates'), {'fields': ('date_joined', 'last_login')}),
    )

    def get_object(self):
        return self.request.user

    def has_read_perm(self):
        return self.request.user.has_perm(f"{self.app_label}.view_user")

    def has_change_perm(self):
        return self.request.user.has_perm(f"{self.app_label}.change_user")

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['fieldsets'] = self.fieldsets
        return context

    def get_success_url(self):
        opts = self.object._meta  # noqa
        current_path = urlquote(self.request.path)
        msg_dict = {
            "name": opts.verbose_name,
            "obj": format_html('<a href="{}">{}</a>', current_path, self.object),
        }
        if "_save" in self.request.POST:
            message = _("The {name} “{obj}” was changed successfully.")
            success_url = reverse("accounts:home")
        else:
            message = _("The {name} “{obj}” was changed successfully. "
                        "You may edit it again below.")
            success_url = current_path
        messages.success(self.request, format_html(message, **msg_dict))
        return success_url

    @method_decorator(csrf_protect)
    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        if not self.has_read_perm():
            raise PermissionDenied
        return super().dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        if not self.has_change_perm():
            messages.error(
                request, _("You don't have permission to edit this profile."))
            return redirect(
                reverse_lazy(
                    "accounts:profile", kwargs={"pk": self.request.user.pk.__str__()})
            )
        return super().post(request, *args, **kwargs)



from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import ReadOnlyPasswordHashField, UsernameField
from django.utils.translation import gettext_lazy as _

User = get_user_model()


class ProfileForm(forms.ModelForm):
    password = ReadOnlyPasswordHashField(
        label=_("Password"),
        help_text=_(
            "Raw passwords are not stored, so there is no way to see "
            "the user’s password."
        ),
    )
    last_login = forms.DateTimeField(
        label=_("Last login"),
        widget=forms.DateTimeInput(attrs={'readonly': 'readonly'})
    )
    date_joined = forms.DateTimeField(
        label=_("Date joined"),
        widget=forms.DateTimeInput(attrs={'readonly': 'readonly'})
    )

    class Meta:
        model = User
        field_classes = {"username": UsernameField}
        exclude = [
            "is_active",
            "is_staff",
            "is_superuser",
            "groups",
            "user_permissions",
        ]



from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.dispatch import receiver

User = get_user_model()


@receiver(models.signals.post_save, sender=User)
def assign_user_permission(sender, instance: User, created: bool, **kwargs):
    if created:
        # Get the content type for the User model
        user_content_type = ContentType.objects.get_for_model(User)

        # Get the specific permissions you want to assign (view, change, etc.)
        view_permission = Permission.objects.get(codename='view_user',
                                                 content_type=user_content_type)
        change_permission = Permission.objects.get(codename='change_user',
                                                   content_type=user_content_type)

        # Assign permissions to the user instance
        instance.user_permissions.add(view_permission, change_permission)
        instance.save()



import logging
from typing import Any, Dict, List

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ImproperlyConfigured
from django.test.signals import setting_changed
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _

logger = logging.getLogger(__name__)

User = get_user_model()

DEFAULTS = {
    "SITE_TITLE": _("NOCOEM"),
    "SITE_HEADER": _("NOCOEM"),
    "LOGIN_URL": "/accounts/login/",
    "LOGIN_REDIRECT_URL": "/accounts/",
    "ADAPTER": "accounts.adapter.DefaultUserAdapter",
    "HOME_TEMPLATE": "accounts/home.html",
    "LOGIN_TEMPLATE": "accounts/login_form.html",
    "LOGOUT_TEMPLATE": "accounts/logged_out.html",
    "PROFILE_TEMPLATE": "accounts/profile_form.html",
    "PASSWORD_CHANGE_TEMPLATE": "accounts/password_change_form.html",
    "PASSWORD_CHANGE_DONE_TEMPLATE": "accounts/password_change_form.html",
    "PASSWORD_RESET_TEMPLATE": "accounts/password_reset_form.html",
    "PASSWORD_RESET_EMAIL_TEMPLATE": "accounts/password_reset_email.html",
    "PASSWORD_RESET_DONE_TEMPLATE": "accounts/password_reset_done.html",
    "PASSWORD_RESET_CONFIRM_TEMPLATE": "accounts/password_reset_confirm.html",
    "PASSWORD_RESET_COMPLETE_TEMPLATE": "accounts/password_reset_complete.html",
}

# List of settings that may be in string import notation.
IMPORT_STRINGS = [
    "ADAPTER",
]


def perform_import(val, setting_name):
    """
    If the given setting is a string import notation,
    then perform the necessary import or imports.
    """
    if val is None:
        return None
    elif isinstance(val, str):
        return import_from_string(val, setting_name)
    elif isinstance(val, (list, tuple)):
        return [import_from_string(item, setting_name) for item in val]
    return val


def import_from_string(val, setting_name):
    """
    Attempt to import a class from a string representation.
    """
    try:
        return import_string(val)
    except ImportError as e:
        msg = "Could not import '%s' for API setting '%s'. %s: %s." % (
            val, setting_name, e.__class__.__name__, e)
        raise ImportError(msg)


class Settings:
    def __init__(
            self,
            user_settings: Dict[str, Any] = None,
            defaults: Dict[str, Any] = None,
            import_strings: List[str] = None
    ):
        if user_settings:
            self._user_settings = user_settings
        self.defaults = defaults or DEFAULTS
        self.import_strings = import_strings or IMPORT_STRINGS
        self._cached_attrs = set()

    @property
    def user_settings(self):
        if not hasattr(self, '_user_settings'):
            try:
                self._user_settings = getattr(settings, 'ACCOUNTS', {})
            except ImproperlyConfigured:
                raise AttributeError("Accounts setting not found")
        return self._user_settings

    def __getattr__(self, attr: str) -> Any:
        if attr not in self.defaults:
            raise AttributeError("Invalid ACCOUNTS setting: '%s'" % attr)

        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]

        # Coerce import strings into classes
        if attr in self.import_strings:
            val = perform_import(val, attr)

        # Cache the result
        self._cached_attrs.add(attr)
        setattr(self, attr, val)
        return val

    def reload(self) -> None:
        for attr in self._cached_attrs:
            delattr(self, attr)
        self._cached_attrs.clear()
        if hasattr(self, '_user_settings'):
            delattr(self, '_user_settings')


acc_settings = Settings(None, DEFAULTS, IMPORT_STRINGS)
settings.LOGIN_URL = acc_settings.LOGIN_URL
settings.LOGIN_REDIRECT_URL = acc_settings.LOGIN_REDIRECT_URL


def reload_acc_settings(*_: Any, **kwargs: Any) -> None:
    setting = kwargs["setting"]
    if setting == "ACCOUNTS":
        acc_settings.reload()
    settings.LOGIN_URL = acc_settings.LOGIN_URL
    settings.LOGIN_REDIRECT_URL = acc_settings.LOGIN_REDIRECT_URL


setting_changed.connect(reload_acc_settings)



from django.contrib.auth import views
from django.urls import path, reverse_lazy
from django.views.i18n import JavaScriptCatalog

from .settings import acc_settings
from .views import (
    MyLoginView,
    HomeView,
    ProfileView,
)

app_name = "accounts"
urlpatterns = [
    path("",
         HomeView.as_view(
             template_name=acc_settings.HOME_TEMPLATE),
         name="home"),
    path("login/",
         MyLoginView.as_view(
             template_name=acc_settings.LOGIN_TEMPLATE),
         name="login"),
    path("logout/",
         views.LogoutView.as_view(
             template_name=acc_settings.LOGOUT_TEMPLATE),
         name="logged_out"),
    path("profile/<pk>/change/",
         ProfileView.as_view(
             template_name=acc_settings.PROFILE_TEMPLATE),
         name="profile"),
    path("profile/<pk>/password/",
         views.PasswordChangeView.as_view(
             template_name=acc_settings.PASSWORD_CHANGE_TEMPLATE,
             success_url=reverse_lazy("accounts:password_change_done"), ),
         name="password"),
    path("password_change/",
         views.PasswordChangeView.as_view(
             template_name=acc_settings.PASSWORD_CHANGE_TEMPLATE,
             success_url=reverse_lazy("accounts:password_change_done"), ),
         name="password_change"),
    path("password_change/done/",
         views.PasswordChangeDoneView.as_view(
             template_name=acc_settings.PASSWORD_CHANGE_DONE_TEMPLATE),
         name="password_change_done", ),
    path("password_reset/",
         views.PasswordResetView.as_view(
             email_template_name=acc_settings.PASSWORD_RESET_EMAIL_TEMPLATE,
             success_url=reverse_lazy("accounts:password_reset_done"),
             template_name=acc_settings.PASSWORD_RESET_TEMPLATE, ),
         name="password_reset"),
    path("password_reset/done/",
         views.PasswordResetDoneView.as_view(
             template_name=acc_settings.PASSWORD_RESET_DONE_TEMPLATE),
         name="password_reset_done", ),
    path("reset/<uidb64>/<token>/",
         views.PasswordResetConfirmView.as_view(
             template_name=acc_settings.PASSWORD_RESET_CONFIRM_TEMPLATE,
             success_url=reverse_lazy("accounts:password_reset_complete"), ),
         name="password_reset_confirm", ),
    path(
        "reset/done/",
        views.PasswordResetCompleteView.as_view(
            template_name=acc_settings.PASSWORD_RESET_COMPLETE_TEMPLATE),
        name="password_reset_complete", ),
    path("jsi18n/", JavaScriptCatalog.as_view(), name="jsi18n"),
]
